//******************************************************************************
//* Copyright (c) 2007 Ford Motor Company. All Rights Reserved.
//* Original author:  Ford Motor Company J2EE Center of Excellence
//*
//* Workfile:	Main.java
//* Revision:	0.0.1	18-Jan-07	manstis1	Initial Version
//*
//******************************************************************************
package com.ford.cosmos.jboss.rules;

import java.io.InputStreamReader;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.audit.WorkingMemoryFileLogger;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;
import org.drools.rule.Package;

import com.ford.cosmos.object.model.AbstractPart;
import com.ford.cosmos.object.model.Attribute;
import com.ford.cosmos.object.model.Attributes;
import com.ford.cosmos.object.model.CostingPart;
import com.ford.cosmos.object.model.Estimate;
import com.ford.cosmos.object.model.Machine;
import com.ford.cosmos.object.model.Material;
import com.ford.cosmos.object.model.Price;
import com.ford.cosmos.object.model.Prices;
import com.ford.cosmos.object.model.Process;

/**
 * TODO: Complete class description JavaDoc
 */
public class PrototypeJBossRules {

    private int machineId = 1;

    private int materialId = 1;

    private int numberOfMachines = 2;

    private int numberOfMaterials = 2;

    private int numberOfPricesPerMachine = 2;

    private int numberOfPricesPerMaterial = 2;

    public static void main(String[] args) {
        PrototypeJBossRules p = new PrototypeJBossRules();
        p.runCAPE();
    }

    private void runCAPE() {

        long parseStart = 0;
        long parseEnd = 0;
        long ruleBaseStart = 0;
        long ruleBaseEnd = 0;
        long executeStart = 0;
        long executeEnd = 0;
        final String rulesDrl = "definitions/rules.plastic.injection.moudling.drl";

        try {

            // Use the JANINI compiler as the JDT one isn't available in WAS
            PackageBuilderConfiguration conf = new PackageBuilderConfiguration();
            conf.setCompiler(PackageBuilderConfiguration.JANINO);

            // Load and parse rule definitions
            parseStart = System.currentTimeMillis();
            PackageBuilder builder = new PackageBuilder(conf);
            InputStreamReader drl = new InputStreamReader(PrototypeJBossRules.class.getResourceAsStream(rulesDrl));
            builder.addPackageFromDrl(drl);
            Package pkg = builder.getPackage();
            parseEnd = System.currentTimeMillis();

            // Populate Rule Base with package
            ruleBaseStart = System.currentTimeMillis();
            RuleBase ruleBase = RuleBaseFactory.newRuleBase();
            ruleBase.addPackage(pkg);
            ruleBaseEnd = System.currentTimeMillis();

            System.out.println("Parsed Rules in " + (parseEnd - parseStart) + " ms.");
            System.out.println("Created RuleBase in " + (ruleBaseEnd - ruleBaseStart) + " ms.");

            // Get a Working Memory in which to process rules
            WorkingMemory wm = ruleBase.newWorkingMemory();
            WorkingMemoryFileLogger logger = new WorkingMemoryFileLogger(wm);
            logger.setFileName("event");

            // Create a Machine (Plastic Injection Moulding)
            Machine machine = this.createMachine(Constants.MACHINE_PLASTIC_INJECTION_MOULDING);
            addMachineToWorkingMemory(wm, machine);

            // Create some other Machines
            for (int i = 0; i < this.numberOfMachines - 1; i++) {
                Machine m = this.createMachine((Math.random()) < 0.5d ? Constants.MACHINE_PLASTIC_INJECTION_MOULDING : Constants.MACHINE_STAMPER);
                addMachineToWorkingMemory(wm, m);
            }

            // Create a Material (Thermoplastic)
            Material material = this.createMaterial(Constants.MATERIAL_THERMOPLASTIC);
            addMaterialToWorkingMemory(wm, material);

            // Create some other Materials
            for (int i = 0; i < this.numberOfMaterials - 1; i++) {
                Material m = this.createMaterial((Math.random()) < 0.5d ? Constants.MATERIAL_THERMOPLASTIC : Constants.MATERIAL_STEEL);
                addMaterialToWorkingMemory(wm, m);
            }
            
            // A complete Estimate object
            Estimate[] estimate = new Estimate[1];
            estimate[0] = Utilities.makeExpertEstimate();
            addEstimateToWorkingMemory(wm, estimate[0]);

            // Trigger rules
            executeStart = System.currentTimeMillis();
            wm.fireAllRules();
            executeEnd = System.currentTimeMillis();
            logger.writeToDisk();

        } catch (Exception e) {
            e.fillInStackTrace().printStackTrace(System.out);
        }

    }

    /**
     * Create a Machine complete with "Tonnage" attribute and a number of Prices
     * 
     * @param type
     *        A Constants.MACHINE_xxxxx constant
     * @return A new Machine
     */
    protected Machine createMachine(String type) {

        Attributes attributes;
        Machine machine = new Machine();
        Date economicLevelCorrect = Utilities.makeDate(1, 1, 2007);
        Date economicLevelIncorrect = Utilities.makeDate(1, 2, 2007);

        // A Machine
        machine.setId(machineId++);
        machine.setDescription(type);
        attributes = machine.getAttributeMap();
        attributes.add(new Attribute(Constants.ATTRIBUTES_MACHINE_TONNAGE, new Integer((int)(Math.random() * 1000))));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MACHINE_PLASTICISING_RATE, new Double(Math.random() * 100)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MACHINE_SCREW_DIAMETER, new Double(Math.random() * 50)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MACHINE_RPM, new Double(Math.random() * 100)));

        // A Price
        machine.addPrice(this.createPrice(economicLevelCorrect));

        // Create some other Prices
        for (int i = 0; i < this.numberOfPricesPerMachine - 1; i++) {
            Price p = this.createPrice((Math.random()) < 0.5d ? economicLevelCorrect : economicLevelIncorrect);
            machine.addPrice(p);
        }

        return machine;
    }

    /**
     * Create a Material complete with a number of Prices
     * 
     * @param type
     *        A Constants.MATERIAL_xxxxx constant
     * @return A new Material
     */
    protected Material createMaterial(String type) {

        Attributes attributes;
        Material material = new Material();
        Date economicLevelCorrect = Utilities.makeDate(1, 1, 2007);
        Date economicLevelIncorrect = Utilities.makeDate(1, 2, 2007);

        // A Material
        material.setId(materialId++);
        material.setDescription(type);
        attributes = material.getAttributeMap();
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_SCREW_SPEED, new Double(Math.random() * 100)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_MOULD_TEMP, new Double(Math.random() * 100)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_MOULD_UPLIFT, new Double(Math.random() * 10)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_THERMAL_CONDUCTIVITY, new Double(Math.random() * 10)));
        double nft = Math.random() * 10;
        double dtul = nft - (Math.random() * 5);
        dtul = dtul > 0 ? dtul : 0;
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_NO_FLOW_TEMP, new Double(nft)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_DTUL, new Double(dtul)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_MELT_TEMP, new Double(Math.random() * 100)));
        attributes.add(new Attribute(Constants.ATTRIBUTES_MATERIAL_MELT_UPLIFT, new Double(Math.random() * 10)));
        
        // A Price
        material.addPrice(this.createPrice(economicLevelCorrect));

        // Create some other Prices
        for (int i = 0; i < this.numberOfPricesPerMaterial - 1; i++) {
            Price p = this.createPrice((Math.random()) < 0.5d ? economicLevelCorrect : economicLevelIncorrect);
            material.addPrice(p);
        }

        return material;
    }

    /**
     * Create a Price
     * 
     * @param economicLevel
     *        The Economic Level of the Price
     * @return A new Price
     */
    protected Price createPrice(Date economicLevel) {
        Price price = new Price();
        price.setCurrency(Constants.CURRENCY_USD);
        price.setEconomicLevel(economicLevel);
        price.setRate(((float)Math.random() * 100));
        return price;
    }

    /**
     * Add Estimate complete with all child objects to Working Memory
     * 
     * @param wm
     *        Working Memory
     * @param e
     *        Estimate
     */
    private void addEstimateToWorkingMemory(WorkingMemory wm, Estimate e) {

        // Estimate itself
        wm.assertObject(e);

        // The principle Part
        addPartToWorkingMemory(wm, e.getPart());

    }

    /**
     * Add Part complete with all child objects to Working Memory
     * 
     * @param wm
     *        Working Memory
     * @param part
     *        AbstractPart
     */
    private void addPartToWorkingMemory(WorkingMemory wm, AbstractPart part) {

        Iterator itr;

        // The Part itself
        wm.assertObject(part);
        wm.assertObject(part.getBomEntry());

        //Child Processes
        if (part instanceof CostingPart) {
            CostingPart cp = (CostingPart)part;
            itr = cp.getProcesses().iterator();
            while (itr.hasNext()) {
                Process process = (Process)itr.next();
                addAttributesToWorkingMemory(wm, process.getAttributeMap());
                wm.assertObject(process);
            }
        }

        // Child Parts
        itr = part.getParts().iterator();
        while (itr.hasNext()) {
            AbstractPart p = (AbstractPart)itr.next();
            addPartToWorkingMemory(wm, p);
        }

    }

    /**
     * Add Machine complete with child Attributes and Prices to Working Memory
     * 
     * @param wm
     *        Working Memory
     * @param m
     *        Machine
     */
    private void addMachineToWorkingMemory(WorkingMemory wm, Machine m) {
        addAttributesToWorkingMemory(wm, m.getAttributeMap());
        addPricesToWorkingMemory(wm, m.getPrices());
        wm.assertObject(m);
    }

    /**
     * Add Material complete with child Attributes and Prices to Working Memory
     * 
     * @param wm
     *        Working Memory
     * @param m
     *        Material
     */
    private void addMaterialToWorkingMemory(WorkingMemory wm, Material m) {
        addAttributesToWorkingMemory(wm, m.getAttributeMap());
        addPricesToWorkingMemory(wm, m.getPrices());
        wm.assertObject(m);
    }

    /**
     * Add Attributes to Working Memory
     * 
     * @param wm
     *        Working Memory
     * @param a
     *        Attributes
     */
    private void addAttributesToWorkingMemory(WorkingMemory wm, Attributes a) {
        Iterator itr;
        itr = a.entrySet().iterator();
        while (itr.hasNext()) {
            Map.Entry me = (Map.Entry)itr.next();
            wm.assertObject(me.getValue());
        }
    }

    /**
     * Add Prices to Working Memory
     * 
     * @param wm
     *        Working Memory
     * @param ps
     *        prices
     */
    private void addPricesToWorkingMemory(WorkingMemory wm, Prices ps) {
        Iterator itr;
        itr = ps.iterator();
        while (itr.hasNext()) {
            Price p = (Price)itr.next();
            wm.assertObject(p);
        }
    }

}