//******************************************************************************
//* Copyright (c) 2007 Ford Motor Company. All Rights Reserved.
//* Original author:  Ford Motor Company J2EE Center of Excellence
//*
//* Workfile:	Utilities.java
//* Revision:	0.0.1	17-Jan-07	manstis1	Initial Version
//*
//******************************************************************************
package com.ford.cosmos.jboss.rules;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import com.ford.cosmos.object.model.Attribute;
import com.ford.cosmos.object.model.BOMEntry;
import com.ford.cosmos.object.model.CostingPart;
import com.ford.cosmos.object.model.Estimate;
import com.ford.cosmos.object.model.Process;

/**
 * TODO: Complete class description JavaDoc
 */
public class Utilities {

    private static final double LOG10 = Math.log(10);

    public static Date makeDate(int day, int month, int year) {

        Calendar cal = Calendar.getInstance();
        cal.clear();
        cal.set(year, month - 1, day);
        return cal.getTime();

    }

    public static String formatDate(Date date) {
        SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
        return sdf.format(date);
    }

    public static String makeIndent(int indent) {
        StringBuffer sb = new StringBuffer();
        if (indent > 0) {
            while (indent > 0) {
                sb.append("--");
                indent--;
            }
            sb.append("> ");
        }
        return sb.toString();
    }

    public static Estimate makeExpertEstimate() {

        //Estimate
        Estimate estimate = new Estimate();
        estimate.setId(277);
        estimate.setEconomicLevel(Utilities.makeDate(1, 1, 2007));

        //CostedPart 1
        BOMEntry bom1 = new BOMEntry();

        //User has selected a Process
        Process process1 = new Process();
        process1.setId((long)22);
        process1.setDescription(Constants.PROCESS_PLASTIC_INJECTION_MOULDING);

        //User has selected a CostedPart
        CostingPart component1 = new CostingPart();
        component1.setDescription("CostingPart 1");
        component1.setId(841);
        component1.setCurrency(Constants.CURRENCY_USD);
        component1.getAttributeMap().add(new Attribute(Constants.ATTRIBUTES_COMPONENT_WEIGHT, new Double(0.12)));
        component1.getAttributeMap().add(new Attribute(Constants.ATTRIBUTES_COMPONENT_NUMBER_OF_CAVITIES, new Integer(2)));
        component1.getAttributeMap().add(new Attribute(Constants.ATTRIBUTES_COMPONENT_DOES_IT_REQUIRE_HIGH_TOLERANCE, new Boolean(false)));
        double dominantWall = Math.random() * 20;
        component1.getAttributeMap().add(new Attribute(Constants.ATTRIBUTES_COMPONENT_DOMINANT_WALL, new Double(dominantWall)));
        component1.getAttributeMap().add(new Attribute(Constants.ATTRIBUTES_COMPONENT_MAXIMUM_WALL, new Double(dominantWall + (Math.random() * 10))));
        component1.getAttributeMap().add(new Attribute(Constants.ATTRIBUTES_COMPONENT_IS_IT_A_LENS, new Boolean(true)));

        //Relationships
        estimate.setPart(component1);
        component1.setBomEntry(bom1);
        component1.addProcess(process1);

        return estimate;
    }

    /**
     * Calculates a result based upon a linear interpolation between a test
     * range, a result range and an actual value
     * 
     * @param ltr
     *        Lower Test Range
     * @param utr
     *        Upper Test Range
     * @param lrr
     *        Lower Result Range
     * @param urr
     *        Upper Result Range
     * @param v
     *        Actual value
     * @return
     */
    public static double smooth(double ltr, double utr, double lrr, double urr, double v) {
        return lrr + (((v - ltr) / (utr - ltr)) * (urr - lrr));
    }

    /**
     * Calculates a log value to base x
     * 
     * @param n
     *        The number
     * @param base
     *        The base
     * @return
     */
    public static double logx(double n, double base) {
        return Math.log(n) / Math.log(base);
    }

    /**
     * Calculates a log value to base 10
     * 
     * @param n
     *        The number
     * @return
     */
    public static double log10(double n) {
        return Math.log(n) / LOG10;
    }

}